@@ -1,6 +1,6 @@
 package App::Info::HTTPD::Apache;
 
-# $Id: Apache.pm,v 1.18 2002/06/04 01:10:33 david Exp $
+# $Id: Apache.pm,v 1.21 2002/06/05 23:47:13 david Exp $
 
 =head1 NAME
 
@@ -24,14 +24,36 @@ App::Info::HTTPD::Apache - Information about Apache web server
 
 App::Info::HTTPD::Apache supplies information about the Apache web server
 installed on the local system. It implements all of the methods defined by
-App::Info::HTTPD.
+App::Info::HTTPD. Methods that throw errors will throw them only the first
+time they're called. To start over (after, say, someone has installed Apache)
+construct a new App::Info::HTTPD::Apache object to aggregate new metadata.
 
-When it loads, App::Info::HTTPD::Apache searches the file system for the
-F<httpd>, F<apache-perl>, or F<apache> application. If found, the application
-(hereafer referred to as F<httpd>, regardless of how it was actually found to
-be named) will be called to gather the data necessary for each of the methods
-below. If none of the applications can be found, then Apache is assumed not to
-be installed, and each of the methods will return C<undef>.
+=cut
+
+use strict;
+use App::Info::HTTPD;
+use App::Info::Util;
+use vars qw(@ISA $VERSION);
+@ISA = qw(App::Info::HTTPD);
+$VERSION = '0.05';
+
+my $u = App::Info::Util->new;
+
+=head1 CONSTRUCTOR
+
+=head2 new
+
+  my $apache = App::Info::HTTPD::Apache->new(@params);
+
+Returns an App::Info::HTTPD::Apache object. See L<App::Info|App::Info> for a
+complete description of argument parameters.
+
+When called, C<new()> searches the file system for the F<httpd>,
+F<apache-perl>, or F<apache> application. If found, the application (hereafer
+referred to as F<httpd>, regardless of how it was actually found to be named)
+will be called by the object methods below to gather the data necessary for
+each. If F<httpd> cannot be found, then Apache is assumed not to be installed,
+and each of the object methods will return C<undef>.
 
 App::Info::HTTPD::Apache searches for F<httpd> along your path, as defined by
 C<File::Spec->path>. Failing that, it searches the following directories:
@@ -50,6 +72,16 @@ C<File::Spec->path>. Failing that, it searches the following directories:
 
 =item /bin
 
+=item /opt/apache/bin
+
+=item /etc/httpd/bin
+
+=item /etc/apache/bin
+
+=item /home/httpd/bin
+
+=item /home/apache/bin
+
 =item /sw/bin
 
 =item /sw/sbin
@@ -58,18 +90,10 @@ C<File::Spec->path>. Failing that, it searches the following directories:
 
 =cut
 
-use strict;
-use App::Info::HTTPD;
-use App::Info::Util;
-use Carp ();
-use vars qw(@ISA $VERSION);
-@ISA = qw(App::Info::HTTPD);
-$VERSION = '0.04';
+sub new {
+    # Construct the object.
+    my $self = shift->SUPER::new(@_);
 
-my $obj = {};
-my $u = App::Info::Util->new;
-
-do {
     # Find Apache executable.
     my @paths = ($u->path,
       qw(/usr/local/apache/bin
@@ -78,28 +102,20 @@ do {
          /usr/bin
          /usr/sbin
          /bin
+         /opt/apache/bin
+         /etc/httpd/bin
+         /etc/apache/bin
+         /home/httpd/bin
+         /home/apache/bin
          /sw/bin
          /sw/sbin));
 
     my @exes = qw(httpd apache-perl apache);
 
-    $obj->{apache_exe} = $u->first_cat_path(\@exes, @paths);
+    $self->{apache_exe} = $u->first_cat_exe(\@exes, @paths);
+    return $self;
 };
 
-=head1 CONSTRUCTOR
-
-=head2 new
-
-  my $apache = App::Info::HTTPD::Apache->new;
-
-Returns an App::Info::HTTPD::Apache object. Since App::Info::HTTPD::Apache is
-implemented as a singleton class, the same object will be returned every time.
-This ensures that only the minimum number of system calls are made to gather
-the data necessary for the object methods.
-
-=cut
-
-sub new { bless $obj, ref $_[0] || $_[0] }
 
 =head1 OBJECT METHODS
 
@@ -110,7 +126,9 @@ sub new { bless $obj, ref $_[0] || $_[0] }
 
 Returns true if Apache is installed, and false if it is not.
 App::Info::HTTPD::Apache determines whether Apache is installed based on the
-presence or absence of the F<httpd> application on the file system.
+presence or absence of the F<httpd> application on the file system, as found
+when C<new()> constructed the object. If Apache does not appear to be
+installed, then all of the other object methods will return empty values.
 
 =cut
 
@@ -120,8 +138,9 @@ sub installed { return $_[0]->{apache_exe} ? 1 : undef }
 
   my $name = $apache->name;
 
-Returns the name of the application. App::Info::HTTPD::Apache parses the
-name from the system call C<`httpd -v`>.
+Returns the name of the application. App::Info::HTTPD::Apache parses the name
+from the system call C<`httpd -v`>. See the L<version|"version"> method for a
+list of possible errors.
 
 =cut
 
@@ -135,20 +154,20 @@ sub name {
   my $version = $apache->version;
 
 Returns the apache version number. App::Info::HTTPD::Apache parses the version
-number from the system call C<`httpd --v`>. Returns C<undef> if Apache is not
-installed. Emits a warning if Apache is installed but the version number could
-not be parsed.
+number from the system call C<`httpd --v`>. Throws an error if Apache is
+installed but the version number or name could not be parsed.
 
 =cut
 
 sub version {
-    return unless $_[0]->{apache_exe};
-    unless (exists $_[0]->{version}) {
-        $_[0]->{version} = undef;
-        my $version = `$_[0]->{apache_exe} -v`;
+    my $self = shift;
+    return unless $self->{apache_exe};
+    unless (exists $self->{version}) {
+        $self->{version} = undef;
+        my $version = `$self->{apache_exe} -v`;
         unless ($version) {
-            Carp::carp("Failed to find Apache version with " .
-                       "`$_[0]->{apache_exe} -v`");
+            $self->error("Failed to find Apache version with " .
+                         "`$self->{apache_exe} -v`");
             return;
         }
 
@@ -156,15 +175,15 @@ sub version {
         my ($n, $x, $y, $z) = $version =~
           /Server\s+version:\s+([^\/]*)\/(\d+)\.(\d+).(\d+)/;
         unless ($n and defined $x and defined $y and defined $z) {
-            Carp::carp("Failed to parse Apache name and version from string ".
-                       "'$version'");
+            $self->error("Failed to parse Apache name and version from ".
+                         "string '$version'");
             return;
         }
 
-        @{$_[0]}{qw(name version major minor patch)} =
+        @{$self}{qw(name version major minor patch)} =
           ($n, "$x.$y.$z", $x, $y, $z);
     }
-    return $_[0]->{version};
+    return $self->{version};
 }
 
 =head2 major_version
@@ -173,9 +192,8 @@ sub version {
 
 Returns the Apache major version number. App::Info::HTTPD::Apache parses the
 version number from the system call C<`httpd --v`>. For example, if
-C<version()> returns "1.3.24", then this method returns "1". Returns C<undef>
-if Apache is not installed. Emits a warning if Apache is installed but the
-version number could not be parsed.
+C<version()> returns "1.3.24", then this method returns "1". See the
+L<version|"version"> method for a list of possible errors.
 
 =cut
 
@@ -190,9 +208,8 @@ sub major_version {
 
 Returns the Apache minor version number. App::Info::HTTPD::Apache parses the
 version number from the system call C<`httpd --v`>. For example, if
-C<version()> returns "1.3.24", then this method returns "3". Returns C<undef>
-if Apache is not installed. Emits a warning if Apache is installed but the
-version number could not be parsed.
+C<version()> returns "1.3.24", then this method returns "3". See the
+L<version|"version"> method for a list of possible errors.
 
 =cut
 
@@ -207,9 +224,8 @@ sub minor_version {
 
 Returns the Apache patch version number. App::Info::HTTPD::Apache parses the
 version number from the system call C<`httpd --v`>. For example, if
-C<version()> returns "1.3.24", then this method returns "24". Returns C<undef>
-if Apache is not installed. Emits a warning if Apache is installed but the
-version number could not be parsed.
+C<version()> returns "1.3.24", then this method returns "24". See the
+L<version|"version"> method for a list of possible errors.
 
 =cut
 
@@ -224,42 +240,42 @@ sub patch_version {
 
 Returns the HTTPD root directory path. This path is defined at compile time,
 and App::Info::HTTPD::Apache parses it from the system call C<`httpd -V`>.
-Returns C<undef> if Apache is not installed. Emits a warning if Apache is
-installed but the HTTPD root could not be parsed.
+Throws an error if Apache is installed but the HTTPD root could not be parsed.
 
 =cut
 
 sub httpd_root {
-    return unless $_[0]->{apache_exe};
-    unless ($_[0]->{-V}) {
-        $_[0]->{-V} = 1;
+    my $self = shift;
+    return unless $self->{apache_exe};
+    unless ($self->{-V}) {
+        $self->{-V} = 1;
         # Get the compile settings.
-        my $data = `$_[0]->{apache_exe} -V`;
+        my $data = `$self->{apache_exe} -V`;
         unless ($data) {
-            Carp::carp("Unable to extract compile settings from ".
-                       "`$_[0]->{apache_exe} =V`");
+            $self->error("Unable to extract compile settings from ".
+                         "`$self->{apache_exe} -V`");
             return;
         }
 
         # Split out the parts.
         foreach (split /\s*\n\s*/, $data) {
             if (/magic\s+number:\s+(.*)$/i) {
-                $_[0]->{magic_number} = $1;
+                $self->{magic_number} = $1;
             } elsif (/=/) {
                 $_ =~ s/^-D\s+//;
                 $_ =~ s/"$//;
                 my ($k, $v) = split /\s*=\s*"/, $_;
-                $_[0]->{lc $k} = $v;
+                $self->{lc $k} = $v;
             } elsif (/-D/) {
                 $_ =~ s/^-D\s+//;
-                $_[0]->{lc $_} = 1;
+                $self->{lc $_} = 1;
             }
         }
         # Issue a warning if no httpd root was found.
-        Carp::carp("Could not parse HTTPD root from `$_[0]->{apache_exe} -V`")
-          unless $_[0]->{httpd_root};
+        $self->error("Could not parse HTTPD root from " .
+                     "`$self->{apache_exe} -V`") unless $self->{httpd_root};
     }
-    return $_[0]->{httpd_root};
+    return $self->{httpd_root};
 }
 
 =head2 magic_number
@@ -268,8 +284,8 @@ sub httpd_root {
 
 Returns the "Magic Number" for the Apache installation. This number is defined
 at compile time, and App::Info::HTTPD::Apache parses it from the system call
-C<`httpd -V`>. Returns C<undef> if Apache is not installed or if the magic number
-could not be parsed.
+C<`httpd -V`> See the L<httpd_root|"httpd_root"> method for a list of possible
+errors.
 
 =cut
 
@@ -283,13 +299,14 @@ sub magic_number {
   my $compile_option = $apache->compile_option($option);
 
 Returns the value of the Apache compile option C<$option>. The compile option
-is looked up caset-insensitively. All of the Apache compile options are
+is looked up case-insensitively. All of the Apache compile options are
 collected from the system call C<`httpd -V`>. For compile options that contain
-a corresponding value (such as 'SUEXEC_BIN" or "DEFAULT_PIDLOG"),
+a corresponding value (such as "SUEXEC_BIN" or "DEFAULT_PIDLOG"),
 C<compile_option()> returns the value of the option if the option exists. For
 other options, it returns true (1) if the option was included, and
 false(C<undef>) if it was not. Returns C<undef> if Apache is not installed or
-if the option could not be parsed.
+if the option could not be parsed. See the L<httpd_root|"httpd_root"> method
+for a list of possible errors.
 
 See the Apache documentation at L<http://httpd.apache.org/docs-project/> to
 learn about all the possible compile options.
@@ -324,15 +341,16 @@ the HTTPD root directory. Failing that, it looks for the following:
 
 =back
 
-Returns C<undef> if the file cannot be found.
+Throws an error and returns C<undef> if the file cannot be found.
 
 =cut
 
 sub conf_file {
-    return unless $_[0]->{apache_exe};
-    unless (exists $_[0]->{conf_file}) {
-        my $root = $_[0]->httpd_root;
-        my $conf = $_[0]->compile_option('SERVER_CONFIG_FILE');
+    my $self = shift;
+    return unless $self->{apache_exe};
+    unless (exists $self->{conf_file}) {
+        my $root = $self->httpd_root;
+        my $conf = $self->compile_option('SERVER_CONFIG_FILE');
         $conf = $u->file_name_is_absolute($conf) ?
           $conf : $u->catfile($root, $conf) if $conf;
         # Paths to search.
@@ -344,10 +362,10 @@ sub conf_file {
                      "/etc/httpd/httpd.conf",
                      "/etc/httpd/httpd.conf.default");
 
-        $_[0]->{conf_file} = $u->first_file(@paths)
-          or Carp::carp("No server config file found");
+        $self->{conf_file} = $u->first_file(@paths)
+          or $self->error("No server config file found");
     }
-    return $_[0]->{conf_file};
+    return $self->{conf_file};
 }
 
 =head2 user
@@ -355,17 +373,18 @@ sub conf_file {
   my $user = $apache->user;
 
 Returns the name of the Apache user. This value is collected from the Apache
-configuration file as returned by C<conf_file()>. Returns C<undef> if Apache
-isn't installed or the configuration file cannot be found or if the user name
-could not be parsed from the configuration file.
+configuration file as returned by C<conf_file()>. Throws an error and returns
+C<undef>if the configuration file cannot be found or if the user name, group,
+or port number could not be parsed from the configuration file.
 
 =cut
 
 sub user {
-    return unless $_[0]->{apache_exe};
-    unless (exists $_[0]->{user}) {
-        $_[0]->{user} = undef;
-        my $conf = $_[0]->conf_file or return;
+    my $self = shift;
+    return unless $self->{apache_exe};
+    unless (exists $self->{user}) {
+        $self->{user} = undef;
+        my $conf = $self->conf_file or return;
 
         # This is the place to add more regexes to collect stuff from the
         # config file in the future.
@@ -374,21 +393,20 @@ sub user {
                        qr/^\s*Port\s+(.*)$/ );
         my ($usr, $grp, $prt) = $u->multi_search_file($conf, @regexen);
         # Issue a warning if we couldn't find the user and group.
-        Carp::carp("Could not parse user and group from file '$conf'")
+        $self->error("Could not parse user and group from file '$conf'")
           unless $usr && $grp;
-        Carp::carp("Could not parse port from file '$conf'") unless $prt;
+        $self->error("Could not parse port from file '$conf'") unless $prt;
         # Assign them anyway.
-        @{$_[0]}{qw(user group port)} = ($usr, $grp, $prt);
+        @{$self}{qw(user group port)} = ($usr, $grp, $prt);
     }
-    return $_[0]->{user};
+    return $self->{user};
 }
 
 =head2 group
 
 Returns the name of the Apache user group. This value is collected from the
-Apache configuration file as returned by C<conf_file()>. Returns C<undef> if
-Apache isn't installed or the configuration file cannot be found or if the
-group name could not be parsed from the configuration file.
+Apache configuration file as returned by C<conf_file()>. See the
+L<"user"|user> method for a list of possible errors.
 
 =cut
 
@@ -400,9 +418,8 @@ sub group {
 =head2 port
 
 Returns the port number on which Apache listens. This value is collected from
-Apache configuration file as returned by C<conf_file()>. Returns C<undef> if
-Apache isn't installed or the configuration file could not be found or if the
-port number could not be parsed from the configuration file.
+Apache configuration file as returned by C<conf_file()>. See the
+L<"user"|user> method for a list of possible errors.
 
 =cut
 
@@ -417,19 +434,21 @@ sub port {
 
 Returns the Apache binary directory path. App::Info::HTTPD::Apache simply
 looks for the F<bin> directory under the HTTPD root directory, as returned by
-C<httpd_root()>. Returns C<undef> if Apache is not installed or if the bin
-directory could not be found.
+C<httpd_root()>. Throws an error and eturns C<undef> if Apache is not
+installed or if the bin directory could not be found.
 
 =cut
 
 sub bin_dir {
+    return unless $_[0]->{apache_exe};
     unless (exists $_[0]->{bin_dir}) {
-        $_[0]->{bin_dir} = undef;
         my $root = $_[0]->httpd_root || return;
         if (my $dir = $u->first_cat_path('bin', $root)) {
             $_[0]->{bin_dir} = $dir;
+        } else {
+            $_[0]->error("Could not find bin directory");
+            $_[0]->{bin_dir} = undef;
         }
-
     }
     return $_[0]->{bin_dir};
 }
@@ -440,17 +459,19 @@ sub bin_dir {
 
 Returns the Apache include directory path. App::Info::HTTPD::Apache simply
 looks for the F<include> or F<inc> directory under the F<httpd_root>
-directory, as returned by C<httpd_root()>. Returns C<undef> if Apache is not
-installed or if the inc directory could not be found.
+directory, as returned by C<httpd_root()>. Throws an error and eturns C<undef>
+if Apache is not installed or if the inc directory could not be found.
 
 =cut
 
 sub inc_dir {
     unless (exists $_[0]->{inc_dir}) {
-        $_[0]->{inc_dir} = undef;
         my $root = $_[0]->httpd_root || return;
         if (my $dir = $u->first_cat_path(['include', 'inc',], $root)){
             $_[0]->{inc_dir} = $dir;
+        } else {
+            $_[0]->error("Could not find inc directory");
+            $_[0]->{inc_dir} = undef;
         }
     }
     return $_[0]->{inc_dir};
@@ -461,21 +482,24 @@ sub inc_dir {
   my $lib_dir = $apache->lib_dir;
 
 Returns the Apache library directory path. App::Info::HTTPD::Apache simply
-looks for the F<lib>, F<modules>, or F<libexec> directory under the
-HTTPD root> directory, as returned by C<httpd_root()>. Returns C<undef> if
-Apache is not installed or if the lib directory could not be found.
+looks for the F<lib>, F<modules>, or F<libexec> directory under the HTTPD
+root> directory, as returned by C<httpd_root()>. Throws an error and returns
+C<undef> if Apache is not installed or if the lib directory could not be
+found.
 
 =cut
 
 sub lib_dir {
     unless (exists $_[0]->{lib_dir}) {
-        $_[0]->{lib_dir} = undef;
         my $root = $_[0]->httpd_root || return;
         if (my $dir = $u->first_cat_path(['lib', 'modules', 'libexec'], $root)){
             $_[0]->{lib_dir} = $dir;
         } elsif ($u->first_dir('/usr/lib/apache/1.3')) {
             # The Debian way.
             $_[0]->{lib_dir} = '/usr/lib/apache/1.3';
+        } else {
+            $_[0]->error("Could not find lib direcory");
+            $_[0]->{lib_dir} = undef;
         }
     }
     return $_[0]->{lib_dir};
@@ -487,8 +511,8 @@ sub lib_dir {
 
 Returns the Apache shared object library directory path. Currently, this
 directory is assumed to be the same as the lib directory, so this method is
-simply an alias for C<lib_dir>. Returns C<undef> if Apache is not installed or
-if the lib directory could not be found.
+simply an alias for C<lib_dir>. Throws an error and returns C<undef> if Apache
+is not installed or if the lib directory could not be found.
 
 =cut
 
@@ -500,19 +524,20 @@ if the lib directory could not be found.
 Returns a list (in an array context) or an anonymous array (in a scalar
 reference) of all of the modules statically compiled into Apache. These are
 collected from the system call C<`httpd -l`>. If Apache is not installed,
-C<static_mods()> returns an empty list in an array reference, or an empty
-anonymous array in a scalar context.
+C<static_mods()> throws an error and returns an empty list in an array context
+or an empty anonymous array in a scalar context.
 
 =cut
 
 sub static_mods {
-    return unless $_[0]->{apache_exe};
-    unless (exists $_[0]->{static_mods}) {
-        $_[0]->{static_mods} = undef;
-        my $data = `$_[0]->{apache_exe} -l`;
+    my $self = shift;
+    return unless $self->{apache_exe};
+    unless (exists $self->{static_mods}) {
+        $self->{static_mods} = undef;
+        my $data = `$self->{apache_exe} -l`;
         unless ($data) {
-            Carp::carp("Unable to extract needed data from ".
-                       "`$_[0]->{apache_exe} =l`");
+            $self->error("Unable to extract needed data from ".
+                         "`$self->{apache_exe} =l`");
             return;
         }
 
@@ -520,20 +545,21 @@ sub static_mods {
         my @mods;
         while ($data =~ /^\s*(\w+)\.c\s*$/mg) {
             push @mods, $1;
-            $_[0]->{mod_so} = 1 if $1 eq 'mod_so';
-            $_[0]->{mod_perl} = 1 if $1 eq 'mod_perl';
+            $self->{mod_so} = 1 if $1 eq 'mod_so';
+            $self->{mod_perl} = 1 if $1 eq 'mod_perl';
         }
-        $_[0]->{static_mods} = \@mods;
+        $self->{static_mods} = \@mods;
     }
-    return unless $_[0]->{static_mods};
-    return wantarray ? @{$_[0]->{static_mods}} : $_[0]->{static_mods};
+    return unless $self->{static_mods};
+    return wantarray ? @{$self->{static_mods}} : $self->{static_mods};
 }
 
 =head2 mod_so
 
 Boolean method that returns true when mod_so has been compiled into Apache,
 and false if it has not. The presence or absence of mod_so is determined by
-the system call C<`httpd -l`>. Returns false if Apache has not been installed.
+the system call C<`httpd -l`>. See the L<"static_mods"|static_mods> method for
+a list of possible errors.
 
 =cut
 
@@ -546,8 +572,8 @@ sub mod_so {
 
 Boolean method that returns true when mod_perl has been statically compiled
 into Apache, and false if it has not. The presence or absence of mod_perl is
-determined by the system call C<`httpd -l`>. Returns false if Apache has not
-been installed.
+determined by the system call C<`httpd -l`>. See the
+L<"static_mods"|static_mods> method for a list of possible errors.
 
 =cut
 
@@ -583,8 +609,9 @@ __END__
 
 It's likely that a lot more can be done to collect data about Apache. The
 methodology for determining the lib, inc, bin, and so_lib directories in
-particular may be considered rather weak. Patches from those who know a great
-deal more about interrogating Apache will be most welcome.
+particular may be considered rather weak. And the Port number can be specified
+multiple ways (and times!) in an Apache configuration file. Patches from those
+who know a great deal more about interrogating Apache will be most welcome.
 
 =head1 TO DO
 
